/*
 * @author Bobby Kramer, released under GPL v 2.0
 */
package ulamgraph;

import java.applet.Applet;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Timer;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import org.nfunk.jep.JEP;

public class GraphNum extends JPanel {
	private static final long serialVersionUID = 1L;
	public static final int HEIGHT = 600;
	public static final int WIDTH = 700;
	private int NODE_WIDTH = 4;
	private int PANEL_HALF_WIDTH = (WIDTH - 130) / 2;
	private boolean showPrimes;
	private ArrayList nodeList;
	private int nodeCount;
	private int startNum; // used for if they start less than or greater than
							// 1..
	private Direction d = new Direction();
	private Color primeColor, eq_1_Color, eq_2_Color, overlap_Color;

	public GraphNum(int _nodeCount) {
		this.eq_1_Color = Color.RED;
		this.eq_2_Color = Color.YELLOW;
		this.primeColor = Color.GREEN;
		this.overlap_Color = Color.BLUE;
		this.showPrimes = true;
		this.startNum = 0;
		this.setupSystem(_nodeCount);
		addMouseListener();
	}

	// node count: used for how many nodes are going to be present in this zoom
	// view.
	public void setupSystem(int _nodeCount) {
		this.nodeList = new ArrayList();
		this.nodeCount = _nodeCount;
		this.SetupNodes();

	}

	public void ZoomOut(boolean toZoomOut) {
		if (toZoomOut == true) {
			if (this.NODE_WIDTH >= 2) {
				this.NODE_WIDTH /= 2;
				setupSystem(ZoomHelper.getNum(NODE_WIDTH)
						* ZoomHelper.getNum(NODE_WIDTH));
			}
		} else {
			if (this.NODE_WIDTH <= 64) {
				this.NODE_WIDTH *= 2;
				setupSystem(ZoomHelper.getNum(NODE_WIDTH)
						* ZoomHelper.getNum(NODE_WIDTH));
			}
		}
	}

	public void paint(Graphics g) {
		for (int a = 0; a < this.nodeCount; a++) {
			Node n = (Node) (this.nodeList.get(a));
			if (a == 0) {// the middle node
				g.setColor(Color.BLACK);
			} else if (a == 1) {// the first node
				g.setColor(Color.BLUE);
			} else if ((n.getEQ1() && n.isPrime() && this.showPrimes == true)
					|| (n.getEQ2() && n.isPrime() && this.showPrimes == true)) {
				g.setColor(overlap_Color);
			} else if (n.isPrime() && this.showPrimes == true) { // &&primes.isSelected()){
				g.setColor(primeColor);
			} else if (n.getEQ1() && n.getEQ2()) {
				g.setColor(overlap_Color);
			} else if (n.getEQ1()) {
				g.setColor(eq_1_Color);
			} else if (n.getEQ2()) {
				g.setColor(eq_2_Color);
			} else {
				g.setColor(Color.DARK_GRAY);
			}
			g.fillRect(n.getX() * NODE_WIDTH + PANEL_HALF_WIDTH + NODE_WIDTH
					/ 2, n.getY() * NODE_WIDTH + PANEL_HALF_WIDTH + NODE_WIDTH
					/ 2, NODE_WIDTH, NODE_WIDTH);
			if (this.NODE_WIDTH > 2) {
				g.setColor(Color.black);
				g.drawRect(n.getX() * NODE_WIDTH + PANEL_HALF_WIDTH
						+ NODE_WIDTH / 2, n.getY() * NODE_WIDTH
						+ PANEL_HALF_WIDTH + NODE_WIDTH / 2, NODE_WIDTH,
						NODE_WIDTH);
			}
			// g.drawString(n.getNumber().toString(), a*5, a*5);
		}
	}

	public void SetupNodes() {
		int count = 2;
		int tmpcount = 1;
		Node n;
		int tmpX = 1;
		int tmpY = -1;
		d.setDirection(d.MOVING_LEFT);
		// setup the first node here, because otherway the directions will get
		// mixed up
		n = new Node(startNum);
		n.setLocation(0, 0);
		nodeList.add((Object) n);
		for (int a = 1; a < this.nodeCount; a++) {
			n = new Node(a + startNum);

			if (d.getDirection() == d.MOVING_LEFT) {
				n.setLocation(tmpX, tmpY);
				tmpX--;
				if ((count) == tmpcount) {
					d.setDirection(d.MOVING_DOWN);
					tmpcount = 0;
				}
			} else if (d.getDirection() == d.MOVING_DOWN) {
				n.setLocation(tmpX, tmpY);
				tmpY++;
				if (count == tmpcount) {
					d.setDirection(d.MOVING_RIGHT);
					tmpcount = 0;
				}
			} else if (d.getDirection() == d.MOVING_RIGHT) {
				n.setLocation(tmpX, tmpY);
				tmpX++;
				if (count == tmpcount) {
					d.setDirection(d.MOVING_UP);
					tmpcount = 0;
				}
			} else if (d.getDirection() == d.MOVING_UP) {
				n.setLocation(tmpX, tmpY);
				tmpY--;
				if (count == tmpcount) {
					d.setDirection(d.MOVING_LEFT);
					tmpcount = 0;
					count += 2;
					tmpY--;
					tmpX++;
				}
			}
			if (isPrime(n.getNumber())) {
				n.setIsPrime(true);
			} else {
				n.setIsPrime(false);
			}
			nodeList.add((Object) n);
			tmpcount++;
		}
	}

	/**
	 * Resets the nodes so that everything is either t
	 */
	public void resetGraph() {
		for (int i = 0; i < this.nodeList.size(); i++) {
			Node n = (Node) (this.nodeList.get(i));
			n.setEQ1(false);
			n.setEQ2(false);
		}
	}

	/**
	 * Parses a result of a equation ran from JEP for equation 1
	 * 
	 * @param result
	 *            the result that JEP returned
	 */
	public void parseEQ1Result(Object result) {
		int StringOutput;
		double testDecimal; // the test val we use to make sure its in the
							// NATURAL numbers
		double output;
		if (result != null) {
			output = Double.parseDouble(result.toString());
			testDecimal = Math.floor(output); // test to make
			if (output - testDecimal != 0) {
				return;
			}
			StringOutput = (int) (output);
			if ((StringOutput) < this.nodeCount + startNum
					&& StringOutput > startNum) {
				Node n = (Node) (this.nodeList.get(StringOutput - startNum));
				n.setEQ1(true);
				return;
			}
			// else if(StringOutput >= this.nodeCount || this.nodeCount <
			// 0){//terminate loop
			// a = this.nodeCount; //not sure if htis is needed?
			// }
		}
	}

	/**
	 * Parses a result of a equation ran from JEP for equation 2
	 * 
	 * @param result
	 *            the result that JEP returned
	 */
	public void parseEQ2Result(Object result) {
		int StringOutput;
		double testDecimal;
		double output;
		if (result != null) {
			output = Double.parseDouble(result.toString());
			testDecimal = Math.floor(output); // test to make
			if (output - testDecimal != 0) {
				return;
			}
			StringOutput = (int) (output);
			if ((StringOutput) < this.nodeCount + startNum
					&& StringOutput > startNum) {
				Node n = (Node) (this.nodeList.get(StringOutput - startNum));
				n.setEQ2(true);
				return;
			}
			// else if(StringOutput >= this.nodeCount || this.nodeCount <
			// 0){//terminate loop
			// a = this.nodeCount; //not sure if htis is needed?
			// }
		}
	}

	boolean isPrime(int num_check) {
		int div;
		if (num_check <= 1) {
			return false;
		}
		if (num_check > 0) {
			int maxToTry = (int) Math.sqrt(num_check);
			if (num_check % 2 == 0) {
				return false;
			} else {
				for (div = 3; div <= maxToTry; div += 2) {
					if (num_check % div == 0) {
						return false;
					}
				}
			}
		} else {

		}
		return true;
	}

	public int getNodeCount() {
		return this.nodeCount;
	}

	public int getStartNum() {
		return this.startNum;
	}

	/**
	 * These colors set what is drawn for a given color
	 */
	public void setPrimeColor(Color c) {
		this.primeColor = c;

	}

	public void setEQ1Color(Color c) {
		this.eq_1_Color = c;
	}

	public void setEQ2Color(Color c) {
		this.eq_2_Color = c;
	}

	public void setOverlapColor(Color c) {
		this.overlap_Color = c;
	}

	public Color getPrimeColor() {
		return this.primeColor;

	}

	public Color getEQ1Color() {
		return this.eq_1_Color;
	}

	public Color getEQ2Color() {
		return this.eq_2_Color;
	}

	public Color getOverlapColor() {
		return this.overlap_Color;
	}

	/**
	 * Controls whether primes are shown or not
	 */
	public void setShowPrimes(boolean val) {
		this.showPrimes = val;
	}

	/**
	 * Sets the starting number
	 */
	public void setStartNum(int num) {
		this.startNum = num;
		setupSystem(ZoomHelper.getNum(NODE_WIDTH)
				* ZoomHelper.getNum(NODE_WIDTH));
	}

	public void findNumber(int x, int y) {

		int tmpx = x;
		int tmpy = y;
		tmpx = PANEL_HALF_WIDTH;
		tmpy -= PANEL_HALF_WIDTH;
		tmpx /= this.NODE_WIDTH;
		tmpy /= this.NODE_WIDTH;
		System.out.println("Tmp x and y are: " + tmpx + " " + tmpy);
		int nodeIndex = findNodeIndex(tmpx, tmpy);
		Node n = (Node) ((Object) this.nodeList.get(nodeIndex));
		System.out.println(n.getNumber() + " IS THE NUMBER");
		// tmp
		// if()
		// g.fillRect(n.getX()*NODE_WIDTH + PANEL_HALF_WIDTH + NODE_WIDTH/2,
		// n.getY()*NODE_WIDTH + PANEL_HALF_WIDTH + NODE_WIDTH/2, NODE_WIDTH,
		// NODE_WIDTH);

	}

	public int findNodeIndex(int x, int y) {
		for (int a = 0; a < this.nodeCount; a++) {
			Node n = (Node) (this.nodeList.get(a));
			if (n.getX() == x && n.getY() == y) {
				// a = this.nodeCount;
				// System.out.println
				return a;
			}
		}
		return 0;
	}

	public void addMouseListener() {
		this.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent me) {
				System.out.println("MOUSE WAS CLICKED");
				if (me.getPoint().x < PANEL_HALF_WIDTH) {
					// findNumber(me.getPoint().x, me.getPoint().y);
				}
			}

			public void mouseEntered(MouseEvent me) {
			}

			public void mousePressed(MouseEvent me) {
			}

			public void mouseReleased(MouseEvent me) {
			}

			public void mouseExited(MouseEvent me) {
			}
		});
	}

}
